from .tc01_spl import Suphalak
from .tc02_mlt import Malet
from .tc03_wcm import WichienMaat
from .tc04_kmn import KhaoManee
from .tc05_knj import Kornja

__all__ = [
    "Suphalak",
    "Malet",
    "WichienMaat",
    "KhaoManee",
    "Kornja",
]

__version__ = "0.2.7"
